#include <Editor/LayerView.hpp>

LayerView::LayerView(): JUI::Window() {
    this->Title("Layers");
    Name("LayerView");

    scroller = new JUI::ScrollingRect(this->Content());
    scroller->Size({100_percent, 100_percent});

    layout = new JUI::VerticalListLayout(this->Content());
    layout->LayoutOrder(JUI::LayoutOrder::V::TOP);
}

LayerView::LayerView(Widget *parent): LayerView() {
    Parent(parent);
}

void LayerView::ClearComponents() {
    for (auto entry : entry_elements) {
        entry->Parent(nullptr);
        delete entry;
    }

    entry_elements.clear();
}

void LayerView::UpdateComponents(const Level *level) {
    stored_level_data = level;

    ClearComponents();

    for (auto layer : level->layers)
    {
        bool is_bottom_layer_in_stack = false;
        bool is_top_layer_in_stack = false;

        if (layer->order == 0) is_top_layer_in_stack = true;
        if (layer->order == level->layers.size()-1) is_bottom_layer_in_stack = true;

        auto* rect = new JUI::Rect(layout);
        rect->LayoutOrder(layer->order);
        rect->Size({100_percent, JUI::UDim(layer_entry_height, 0)});
        auto* vis_chk = new JUI::Checkbox(rect);
        vis_chk->Size({JUI::UDim(layer_entry_height, 0), JUI::UDim(layer_entry_height, 0)});
        auto* label = new JUI::TextButton(rect);
        label->Name("Label");
        label->Position({JUI::UDim(layer_entry_height, 0), 0_px});
        label->Size({100_percent-JUI::UDim(layer_entry_height*2, 0), JUI::UDim(layer_entry_height, 0)});
        label->Content(layer->name);
        label->TextColor(Colors::Black);

        label->OnReleaseEvent += [this, label, layer] (auto a, auto b, auto c) mutable {
            ActiveLayerSelected.Invoke(layer->name);
            label->BaseBGColor(Colors::Gray);
        };

        vis_chk->OnReleaseEvent += [this, layer](auto a, auto b, auto c) mutable {
            LayerVisibilityToggled.Invoke(layer->name, !layer->visible);
        };

        auto* layer_order_shift_box = new JUI::Rect(rect);
        layer_order_shift_box->Size({JUI::UDim(layer_entry_height, 0), JUI::UDim(layer_entry_height, 0)});
        layer_order_shift_box->Position({100_percent-JUI::UDim(layer_entry_height, 0), 0_px});


        auto layer_up = new JUI::TextButton(layer_order_shift_box);
        layer_up->Size({100_percent, 50_percent});
        layer_up->TextColor(Colors::Black);
        layer_up->Content("/\\");
        layer_up->Center();

        layer_up->OnReleaseEvent += [this, layer] (auto a, auto b, auto c) mutable {
            if (layer->order > 0)
                layer->order--;
        };

        if (is_top_layer_in_stack)
            layer_up->Disable();


        auto* layer_down = new JUI::TextButton(layer_order_shift_box);
        layer_down->Size({100_percent, 50_percent});
        layer_down->Content("\\/");
        layer_down->TextColor(Colors::Black);
        layer_down->Position({0_percent, 50_percent});
        layer_down->Center();

        layer_down->OnReleaseEvent += [this, level, layer] (auto a, auto b, auto c) mutable {
            const int layer_count = level->layers.size();
            if (layer->order <= layer_count)
                layer->order++;
        };

        if (is_bottom_layer_in_stack)
            layer_down->Disable();



        entry_elements.push_back(rect);

    }
}


