#include <Editor/TileMetaDialog.hpp>


TileMetaDialog::TileMetaDialog(): JUI::Window() {
    using namespace JUI::UDimLiterals;

    Name("TileMetaDialog");
    Title("Tile Metadata: #0");

    Size({150_px, 60_px});
    AnchorPoint({0.5, 0.5});

    name_label = new JUI::TextRect(this->Content());
    name_label->Size({50_px, 20_px});
    name_label->Center();
    name_label->Content("Tile Name:");
    name_form = new JUI::TextInputForm(this->Content());
    name_form->Size({70_px, 20_px});
    name_form->Position({50_px, 0_px});
    //name_form->SetInputBuffer("MyTile");
    // TODO: autocomplete_text_enabled does nothing.
    name_form->SetAutocompleteTextEnabled(false);
    name_form->SetAutoCompleteText("");
    name_form->Content("Another One");
    name_form->ClearTextOnReturn(false);


    collides_label  = new JUI::TextRect(this->Content());
    collides_label->Size({50_px, 20_px});
    collides_label->Position({0_px, 20_px});
    collides_label->Center();
    collides_label->Content("Collides:");
    collides_form = new JUI::Checkbox(this->Content());
    collides_form->Size({20_px, 20_px});
    collides_form->Position({50_px, 20_px});


    name_form->OnReturn += [&] (const std::string& new_name) {
        NameChanged(new_name);
    };

    collides_form->OnReleaseEvent += [&](auto pos, auto btn, bool value) mutable {
        CollidesChanged(!collides_form->IsChecked());
    };
}

TileMetaDialog::TileMetaDialog(JUI::Widget *parent): TileMetaDialog() {
    this->Parent(parent);
}

void TileMetaDialog::SetTileMetadata(const Tile &tile) {

    cur_tile_id = tile.id;
    cur_tile_name = tile.name;

    name_form->Content(tile.name);
    collides_form->SetChecked(tile.collides);

    Title(std::format("Tile Metadata: {} #{}", tile.name, tile.id));
}

int TileMetaDialog::CurrentTileID() const { return cur_tile_id;}

std::string TileMetaDialog::CurrentTileName() const { return cur_tile_name;}

void TileMetaDialog::OnExit(const Vector2 &MousePos) {
    this->Close();
}
