#include <Format/Tileset.hpp>

Tileset::Tileset():
	tile_width(0),tile_height(0), tile_spacing(0),
	tex_width(0), tex_height(0), rows(0), cols(0),
	tile_count(0), tiles_per_row(0)
{ }


Tileset::Tileset(const std::string& name, const std::filesystem::path& texture_path, int tex_width, int tex_height, int tile_width, int tile_height,
				 int tile_spacing)
{
	this->name = name;
	this->file_path = std::format("{}.tileset", name);

	this->tex_width = tex_width;
	this->tex_height = tex_height;
	this->texture_path = texture_path.string();

	this->tile_width = tile_width;
	this->tile_height = tile_height;
	this->tile_spacing = tile_spacing;

	this->rows;
	this->cols;

	this->rows = J3ML::Math::Floor(this->tex_width / this->tile_width);
	this->cols = J3ML::Math::Floor(this->tex_height / this->tile_height);

	ComputeQuads();
	FillTiles();
}

Tileset::Tileset(const json::value& json)
{

	Deserialize(json);
}

Tileset::Tileset(const std::filesystem::path& filePath)
{

	auto [json, err] = json::parse(read_file_contents(filePath));
	Deserialize(json);
}

void Tileset::Deserialize(const json::value& json)
{
	this->name = json["name"].as_string();
	this->author = json["author"].as_string();
	this->file_path = json["file-path"].as_string();
	this->texture_path = json["texture-path"].as_string();
	this->tile_width = (int)json["tile-width"].number.value();
	this->tile_height = (int)json["tile-height"].number.value();
	this->tile_spacing = (int)json["tile-spacing"].number.value();
	this->tex_width = (int)json["texture-width"].number.value();
	this->tex_height = (int)json["texture-height"].number.value();

	this->rows = J3ML::Math::Floor(this->tex_width / this->tile_width);
	this->cols = J3ML::Math::Floor(this->tex_height / this->tile_height);

	ComputeQuads();
	FillTiles();


	if (json.as_object().contains("tiles"))
	{
		auto tilemeta = json["tiles"].as_array();
		for (json::value& v : tilemeta)
		{
			int id = v["id"].number.value();

			if (id < this->tile_count) {
				tiles[id].persistent = true;

				// TODO: Warning, tile out of tileset ID bounds!
				tiles[id].name = v["name"].as_string();
				if (v.contains("collides"))
					tiles[id].collides = v["collides"].boolean.value_or(true);

			}



			//	this->tiles.push_back(v);
		}
	}

}


json::value Tileset::Serialize() const
{
	json::object tileset;
	tileset["name"] = this->name;
	tileset["file-path"] = this->file_path;
	tileset["texture-path"] = this->texture_path;
	tileset["author"] = this->author;

	tileset["tile-width"] = (float) this->tile_width;
	tileset["tile-height"] = (float) this->tile_height;
	tileset["tile-spacing"] = (float)this->tile_spacing;
	tileset["texture-width"] = (float)this->tex_width;
	tileset["texture-height"] = (float)this->tex_height;

	tileset["first-gid"] = 0.f;
	tileset["rows"] = (float)this->rows;
	tileset["cols"] = (float)this->cols;

	/*json::array quad_array;
        for (int i = 0; i < this->quads.size(); ++i)
        {
            json::array quad;
            quad.push_back((float)quads[i].x);
            quad.push_back((float)quads[i].y);
            quad.push_back((float)quads[i].h);
            quad.push_back((float)quads[i].w);
            quad_array.push_back(quad);
        }

        tileset["quads"] = quad_array;*/

	json::array tiles;
	for (auto& tile : this->tiles)
	{
		if (tile.persistent) {
			json::object tile_json;
			tile_json["name"] = tile.name;
			tile_json["id"] = (float)tile.id;
			tile_json["metadata"] = tile.metadata;
			tile_json["collides"] = json::boolean(tile.collides);
			//json::array quad;
			//quad.push_back((float)tile.quad.x);
			//quad.push_back((float)tile.quad.y);
			//quad.push_back((float)tile.quad.w);
			//quad.push_back((float)tile.quad.h);
			//tile_json["quad"] = quad;
			tiles.push_back(tile_json);
		}
	}
	tileset["tiles"] = tiles;

	return tileset;
}

Vector2i Tileset::IndexToCell(int index) const
{
	return {index % rows, index / rows};
}

int Tileset::CellToIndex(int cx, int cy) const
{
	return cy*rows + cx;
}

void Tileset::ComputeQuads()
{
	quads.reserve(rows*cols);

	for (int i = 0; i < rows*cols; i++) {
		Vector2i cell = IndexToCell(i);
		quads.push_back(Quad{cell.x*tile_width, cell.y*tile_height, tile_width, tile_height});
	}
}
