#include <Colors.hpp>
#include <Re2DLevelAPI.hpp>
#include <fstream>
#include <json.hpp>


Color4 JsonConversions::parse_color(const json::value& v)
{
	if (v.type == json::value_type::string)
		return Color4::FromHex(v.string.value());
	else if (v.type == json::value_type::array)
	{
		auto color_array = v.as_array();
		int r = color_array[0].number.value();
		int g = color_array[1].number.value();
		int b = color_array[2].number.value();
		int a = 255;
		if (color_array.size() == 4)
			a = color_array[3].number.value();
		return Color4(r, g, b, a);
	}
	else if (v.type == json::value_type::object)
	{
		auto color_obj = v.operator std::map<std::string, json::value>();
		int r = color_obj["r"].number.value();
		int g = color_obj["g"].number.value();
		int b = color_obj["b"].number.value();

		int a = 255;

		if (color_obj.contains("a"))
			a = color_obj["a"].number.value();

		return Color4(r, g, b, a);
	}
	return Colors::Transparent;
}

json::value JsonConversions::deparse_color_to_hex(const Color4& color)
{
	return json::string(color.ToHexAlpha());
}

std::vector<std::string> JsonConversions::parse_string_list(const json::value& v)
{
	// TODO: Log an error if the json value is an invalid type.
	if (v.type == json::value_type::string)
	{
		return {v.string.value()};
	}
	else if (v.type == json::value_type::array)
	{
		std::vector<std::string> retval;

		for (auto& token : v.array.value())
			retval.push_back(token.string.value());
		return retval;
	}
	return {};
}

json::value JsonConversions::deparse_string_list(const std::vector<std::string>& list)
{
	json::array strlist;
	for (auto& str : list)
		strlist.push_back(json::string(str));

	return strlist;
}

std::string read_file_contents(const std::filesystem::path& file_path)
{
	std::ifstream file (file_path);
	std::ostringstream sstr;
	sstr << file.rdbuf();
	return sstr.str();
}

bool write_file_contents(const std::filesystem::path& file_path, const std::string& contents)
{
	std::ofstream output(file_path);

	if (output.is_open()) {
		output << contents;
		output.close();
		return true;
	}
	return false;
}

std::vector<std::string> split(const std::string &s, char delim) {
	std::vector<std::string> result;
	std::stringstream ss (s);
	std::string item;

	while (getline (ss, item, delim)) {
		result.push_back (item);
	}

	return result;
}
